import numpy

def rotx(rad):
    return numpy.array([[1,0,0],[0,numpy.cos(rad), -numpy.sin(rad)], [0, numpy.sin(rad), numpy.cos(rad)]])

def roty(rad):
    return numpy.array([[numpy.cos(rad),0,numpy.sin(rad)],[0,1,0], [-numpy.sin(rad), 0, numpy.cos(rad)]])

def rotz(rad):
    return numpy.array([[numpy.cos(rad),-numpy.sin(rad),0],[numpy.sin(rad),numpy.cos(rad), 0], [0, 0, 1]])

def rpy2R(*args):
    """roll, pitch, yaw to SE(3) rotation matrix"""
    r,p,y = (0,0,0)
    if len(args) == 3:
        r,p,y = args
    elif len(args) == 1:
        r,p,y = args[0]
    else:
        raise
    R = numpy.dot(rotz(y),numpy.dot(roty(p),rotx(r)))
    return R

def R2rpy(R):
    r = R2roll(R)
    p = R2pitch(R)
    y = R2yaw(R)
    return numpy.array([r, p, y])

def R2roll(R):
    r = numpy.arctan2(R[2,1],R[2,2])
    return r

def R2pitch(R):
    p = numpy.arctan2(-R[2,0],numpy.sqrt(R[2,1]**2 + R[2,2]**2))
    return p

def R2yaw(R):
    y = numpy.arctan2(R[1,0],R[0,0])
    return y



import numpy
from rot3d import *

def xyzrpy2T(*args):
    """roll, pitch, yaw, x, y, z to SE(3) homogenous transform"""
    x,y,z,r,p,yaw = (0,0,0,0,0,0)
    if len(args) == 2:
        x,y,z = args[0]
        r,p,yaw = args[1]
    elif len(args) == 4:
        if (type(args[0]) == type(list())) or type(args[0]) == type(numpy.array([])):
            x,y,z = args[0]
            r,p,yaw = args[1:4]
        elif (type(args[3]) == type(list())) or type(args[3]) == type(numpy.array([])):
            x,y,z = args[0:3]
            r,p,yaw = args[3]
        else:
            raise
    elif len(args) == 6:
        x,y,z = args[0:3]
        r,p,yaw = args[3:6]
    else:
        raise
    R = rpy2R(r,p,yaw)
    T = numpy.eye(4)
    T[0:3, 0:3] = R
    T[0:3, 3] = [x,y,z]
    return T

def T2xyzrpy(T):
    r, p, yaw = R2rpy(T[0:3, 0:3])
    x, y, z = T[0:3, 3]
    return numpy.array([x, y, z, r, p, yaw])



import numpy
from rot3d import *
from homtrans3d import *

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits.mplot3d import proj3d

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

def plotT(T, title=""):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.set_aspect("equal")
    ax.set_title(title)

    # plot the transform
    pos = T[0:3,3]
    # column space of the rotation matrix is the rotated unit basis frame
    px = T[0:3,0]
    py = T[0:3,1]
    pz = T[0:3,2]
    # hack together equal aspect ratio and scaling the unit vectors to be visible
    points = numpy.array([pos,pos+px,pos+py,pos+pz, [0,0,0], [1,0,0], [0,1,0], [0,0,1]]).T
    X = points[0]
    Y = points[1]
    Z = points[2]
    max_range = numpy.array([X.max()-X.min(), Y.max()-Y.min(), Z.max()-Z.min()]).max()
    scale = max_range/2
    vx = numpy.array([pos,pos + px*scale]).T
    vy = numpy.array([pos,pos + py*scale]).T
    vz = numpy.array([pos,pos + pz*scale]).T
    x = Arrow3D(vx[0],vx[1],vx[2], mutation_scale=20, lw=1, arrowstyle="-|>", color="r")
    y = Arrow3D(vy[0],vy[1],vy[2], mutation_scale=20, lw=1, arrowstyle="-|>", color="b")
    z = Arrow3D(vz[0],vz[1],vz[2], mutation_scale=20, lw=1, arrowstyle="-|>", color="b")
    ax.add_artist(x)
    ax.add_artist(y)
    ax.add_artist(z)

    # plot the basis
    i = Arrow3D([0,scale],[0,0],[0,0], mutation_scale=20, lw=1, arrowstyle="-|>", color="r")
    j = Arrow3D([0,0],[0,scale],[0,0], mutation_scale=20, lw=1, arrowstyle="-|>", color="k")
    k = Arrow3D([0,0],[0,0],[0,scale], mutation_scale=20, lw=1, arrowstyle="-|>", color="k")
    ax.add_artist(i)
    ax.add_artist(j)
    ax.add_artist(k)

    # hack together the bounding box for 3d equal aspect ratio
    # http://stackoverflow.com/questions/13685386/matplotlib-equal-unit-length-with-equal-aspect-ratio-z-axis-is-not-equal-to
    Xb = 0.5*max_range*numpy.mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(X.max()+X.min())
    Yb = 0.5*max_range*numpy.mgrid[-1:2:2,-1:2:2,-1:2:2][1].flatten() + 0.5*(Y.max()+Y.min())
    Zb = 0.5*max_range*numpy.mgrid[-1:2:2,-1:2:2,-1:2:2][2].flatten() + 0.5*(Z.max()+Z.min())
    # Comment or uncomment following both lines to test the fake bounding box:
    for xb, yb, zb in zip(Xb, Yb, Zb):
       ax.plot([xb], [yb], [zb], 'w', alpha=0)

    ax.set_xlabel("x")
    ax.set_ylabel("y")
    ax.set_zlabel("z")

    plt.show()



import numpy
from rot3d import *
from homtrans3d import *
from plotT import *

class SE3:
    def __init__(self, *args):
        """x, y, z, roll, pitch, yaw"""
        if len(args) == 0:
            self.T = numpy.eye(4)
        else:
            self.T = xyzrpy2T(*args)

    def __repr__(self):
        return "SE3 Homogenous Transform \n\
\tx:\t%s \n\
\ty:\t%s \n\
\tz:\t%s \n\
\troll:\t%s \n\
\tpitch:\t%s \n\
\tyaw:\t%s" % tuple(T2xyzrpy(self.T))

    def __str__(self):
        return "SE3(%s, %s, %s, %s, %s, %s)" % tuple(T2xyzrpy(self.T))

    def plot(self, title=""):
        plotT(self.T, title)

    def translate(self, *args):
        move = numpy.array([0,0,0])
        if len(args) == 1:
            move = numpy.array(args[0])
        elif len(args) == 3:
            move = numpy.array(args)
        else:
            raise
        self.T = numpy.dot(self.T, xyzrpy2T(move,0,0,0))
        return self

    def rotate(self, *args):
        rot = numpy.array([0,0,0])
        if len(args) == 1:
            rot = numpy.array(args[0])
        elif len(args) == 3:
            rot = numpy.array(args)
        else:
            raise
        self.T = numpy.dot(self.T, xyzrpy2T(0,0,0,rot))
        return self

    def transform(self, other):
        if not isinstance(other, self.__class__):
            raise
        self.T = numpy.dot(self.T, other.T)
        return self

    # global translation, rotation, transformation
    # use this when you want to transform this coordinate
    # frame based on the global reference frame
    def gTranslate(self, *args):
        move = numpy.array([0,0,0])
        if len(args) == 1:
            move = numpy.array(args[0])
        elif len(args) == 3:
            move = numpy.array(args)
        else:
            raise
        self.T = numpy.dot(xyzrpy2T(move,0,0,0),self.T)
        return self

    def gRotate(self, *args):
        rot = numpy.array([0,0,0])
        if len(args) == 1:
            rot = numpy.array(args[0])
        elif len(args) == 3:
            rot = numpy.array(args)
        else:
            raise
        self.T = numpy.dot(xyzrpy2T(0,0,0,rot),self.T)
        return self

    def gTransform(self, other):
        if not isinstance(other, self.__class__):
            raise
        self.T = numpy.dot(other.T,self.T)
        return self

    def x(self):
        return self.T[0,3]
    def y(self):
        return self.T[1,3]
    def z(self):
        return self.T[2,3]
    def roll(self):
        return R2roll(self.T)
    def pitch(self):
        return R2pitch(self.T)
    def yaw(self):
        return R2yaw(self.T)

    def __mul__(self, other):
        if not isinstance(other, self.__class__):
            raise
        se3 = SE3()
        se3.T = numpy.dot(self.T,other.T)
        return se3

    def __rmul__(self, other):
        if not isinstance(other, self.__class__):
            raise
        se3 = SE3()
        se3.T = numpy.dot(other.T,self.T)
        return se3

    def approxEq(self, other, decimal=2):
        if not isinstance(other, self.__class__):
            raise
        return numpy.all(numpy.round(self.T, decimal) == numpy.round(other.T, decimal))

    def __eq__(self, other):
        return self.approxEq(other, decimal=10)

    def __ne__(self, other):
        return not self.__eq__(other)



from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(
    name='se3',
    version='0.1',
    description='Homogenous transforms in special euclidian 3-space.',
    long_description=readme(),
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Science/Research',
    ],
    keywords='keyword1 keyword2',
    url='https://github.com/ccorcos/se3',
    author='Chet Corcos',
    author_email='ccorcos@gmail',
    license='MIT',
    packages=['se3'],
    install_requires=[
        'numpy',
        'matplotlib'
    ],
    test_suite='nose.collector',
    tests_require=['nose'],
    include_package_data=True,
    zip_safe=False
)
